/*
 RFBLE
 Created on: Aug 24, 2023
 Author: Administrator
 */

#include "stdio.h"
#include "ch32v20x.h"
#include "ch32v20x_gpio.h"
#include "string.h"
#include "los_interrupt.h"
#include "los_sem.h"

/* Global define */
#define TxSize1    32
#define RxSize1    64
#define size(a)    (sizeof(a) / sizeof(*(a)))

/* Global Variable */
u8 TxBuffer1[TxSize1]; /* Send by UART2 */
u8 RxBuffer1[RxSize1] = { 0 }; /* USART2 Using  */
u8 RxBuffer1Handle[RxSize1] = { 0 };
UINT32 g_semId;
static uint16_t rxDataCnt;

VOID RFBLE_Task(VOID);
UINT32 RFBLE_TaskInit(VOID);

static void DMA_TX_INIT(uint16_t cnt) {
    DMA_InitTypeDef DMA_InitStructure = { 0 };
    USART_DMACmd(USART1, USART_DMAReq_Tx, DISABLE); //ʹDMA
    DMA_Cmd(DMA1_Channel4, DISABLE); /* USART2 Tx */
    DMA_DeInit(DMA1_Channel4);
    DMA_InitStructure.DMA_PeripheralBaseAddr = (u32) (&USART1->DATAR); /* USART2->DATAR:0x40004404 */
    DMA_InitStructure.DMA_MemoryBaseAddr = (u32) TxBuffer1;
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
    DMA_InitStructure.DMA_BufferSize = cnt;
    DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
    DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
    DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
    DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
    DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
    DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
    DMA_Init(DMA1_Channel4, &DMA_InitStructure);
    DMA_Cmd(DMA1_Channel4, ENABLE); /* USART2 Tx */
    USART_DMACmd(USART1, USART_DMAReq_Tx, ENABLE); //ʹDMA
    while(DMA_GetFlagStatus(DMA1_FLAG_TC4) == RESET) /* Wait until USART2 TX DMA1 Transfer Complete */
        {
        }
}
void UART1_DMA_Send(u8 *buf, u16 cnt) {
    if (cnt > TxSize1)
        return;
    memcpy(TxBuffer1, buf, cnt);
    DMA_TX_INIT(cnt);
    //DMA_SetCurrDataCounter(DMA1_Channel7, cnt);
    //USART_DMACmd(USART1, USART_DMAReq_Tx, ENABLE);
}

VOID RFBLE_Task(VOID) {
    while(1) {
        //LOS_WAIT_FOREVER
        uint32_t ret=LOS_SemPend(g_semId, 100);
        //printf("taskSampleEntry1 running,task1 SP:%08x\n",__get_SP());
        //handle Received data
        if(ret==LOS_OK)
            UART1_DMA_Send(RxBuffer1Handle,rxDataCnt);
        else if(ret==LOS_ERRNO_SEM_TIMEOUT)
        {
            u8 * RxBuffer1Handle1="hello harmonyOS\r\n";
            UART1_DMA_Send(RxBuffer1Handle1,strlen(RxBuffer1Handle1));
        }
    }

}

static void DMA_RX_INIT(void) {
    DMA_InitTypeDef DMA_InitStructure = { 0 };
    USART_DMACmd(USART1, USART_DMAReq_Rx, DISABLE); //ʹDMA
    DMA_Cmd(DMA1_Channel5, DISABLE); /* USART2 Tx */
    DMA_DeInit(DMA1_Channel5);
    DMA_InitStructure.DMA_PeripheralBaseAddr = (u32) (&USART1->DATAR); /* USART2->DATAR:0x40004404 */
    DMA_InitStructure.DMA_MemoryBaseAddr = (u32) RxBuffer1;
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
    DMA_InitStructure.DMA_BufferSize = RxSize1;
    DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
    DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
    DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
    DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
    DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
    DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
    DMA_Init(DMA1_Channel5, &DMA_InitStructure);
    DMA_Cmd(DMA1_Channel5, ENABLE); /* USART2 Tx */
    USART_DMACmd(USART1, USART_DMAReq_Rx, ENABLE); //ʹDMA
}
static void DMA_INIT(void) {
    DMA_InitTypeDef DMA_InitStructure = { 0 };
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);

    DMA_DeInit(DMA1_Channel4);
    DMA_InitStructure.DMA_PeripheralBaseAddr = (u32) (&USART1->DATAR); /* USART2->DATAR:0x40004404 */
    DMA_InitStructure.DMA_MemoryBaseAddr = (u32) TxBuffer1;
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
    DMA_InitStructure.DMA_BufferSize = TxSize1;
    DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
    DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
    DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
    DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
    DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
    DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
    DMA_Init(DMA1_Channel4, &DMA_InitStructure);

    DMA_DeInit(DMA1_Channel5);
    DMA_InitStructure.DMA_PeripheralBaseAddr = (u32) (&USART1->DATAR);
    DMA_InitStructure.DMA_MemoryBaseAddr = (u32) RxBuffer1;
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
    DMA_InitStructure.DMA_BufferSize = size(RxBuffer1);
    DMA_Init(DMA1_Channel5, &DMA_InitStructure);

    DMA_Cmd(DMA1_Channel4, ENABLE); /* USART2 Tx */
    DMA_Cmd(DMA1_Channel5, ENABLE); /* USART2 Rx */

}
void RFBLE_Init(void) {
    GPIO_InitTypeDef GPIO_InitStructure;
    USART_InitTypeDef USART_InitStructure;
    NVIC_InitTypeDef NVIC_InitStructure = { 0 };
    /* źṹid */

    uint32_t baudrate = 115200;

    /* ź */
    LOS_SemCreate(0, &g_semId);
    RFBLE_TaskInit();

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1 | RCC_APB2Periph_GPIOA,
            ENABLE);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    USART_InitStructure.USART_BaudRate = baudrate;
    USART_InitStructure.USART_WordLength = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits = USART_StopBits_1;
    USART_InitStructure.USART_Parity = USART_Parity_No;
    USART_InitStructure.USART_HardwareFlowControl =
            USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode = USART_Mode_Tx|USART_Mode_Rx;
    DMA_INIT();
    USART_Init(USART1, &USART_InitStructure);

    USART_ITConfig(USART1, USART_IT_IDLE, ENABLE);

    NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);

    USART_Cmd(USART1, ENABLE);
    USART_DMACmd(USART1, USART_DMAReq_Rx, ENABLE); //ʹDMA

}

UINT32 RFBLE_TaskInit(VOID) {
    UINT32 uwRet;
    UINT32 taskID1;
    TSK_INIT_PARAM_S stTask = { 0 };
    stTask.pfnTaskEntry = (TSK_ENTRY_FUNC) RFBLE_Task;
    stTask.uwStackSize = 0X500;
    stTask.pcName = "RFBLE_Task";
    stTask.usTaskPrio = 6;/* high priority */
    uwRet = LOS_TaskCreate(&taskID1, &stTask);
    if (uwRet != LOS_OK) {
        printf("create RFBLE_Task failed\n");
    }
    return LOS_OK;
}



/*********************************************************************
 * @fn      USART2_IRQHandler
 *
 * @brief   This function handles USART1 global interrupt request.
 *
 * @return  none
 */
void USART1_IRQHandler(void) {
    GET_INT_SP();
    HalIntEnter();
    if (USART_GetITStatus(USART1, USART_IT_IDLE) != RESET) {

        USART_DMACmd(USART1, USART_DMAReq_Rx, DISABLE);
        //clear IDLE Flag
        USART_GetFlagStatus(USART1, USART_FLAG_IDLE);
        USART_ReceiveData(USART1);
        rxDataCnt = RxSize1 - DMA_GetCurrDataCounter(DMA1_Channel6);
        memcpy(RxBuffer1Handle, RxBuffer1, rxDataCnt);
        DMA_RX_INIT();//³ʼDMA
        LOS_SemPost(g_semId);
    }

    HalIntExit();
    FREE_INT_SP();
}
